import unittest

from cloudrail.dev_tools.rule_test_utils import create_empty_entity
from cloudrail.knowledge.context.gcp.gcp_environment_context import GcpEnvironmentContext
from cloudrail.knowledge.context.gcp.resources.sql.gcp_sql_database_instance import GcpSqlDatabaseInstance, GcpSqlDBInstanceSettings, GcpSqlDBInstanceSettingsIPConfig
from cloudrail.knowledge.rules.base_rule import RuleResultType
from cloudrail.knowledge.rules.gcp.non_context_aware.sql_database_ssl_required_rule import SqlDatabaseSslRequiredRule


class TestSqlDatabaseSslRequired(unittest.TestCase):
    def setUp(self):
        self.rule = SqlDatabaseSslRequiredRule()

    def test_ssl_required(self):
        # Arrange
        sql = create_empty_entity(GcpSqlDatabaseInstance)
        sql.name = 'name'
        sql.settings = create_empty_entity(GcpSqlDBInstanceSettings)
        sql.settings.ip_configuration = create_empty_entity(GcpSqlDBInstanceSettingsIPConfig)
        sql.settings.ip_configuration.require_ssl = True
        context = GcpEnvironmentContext(sql_database_instances=[sql])
        # Act
        result = self.rule.run(context, {})
        # Assert
        self.assertEqual(RuleResultType.SUCCESS, result.status)

    def test_ssl_not_required(self):
        # Arrange
        sql = create_empty_entity(GcpSqlDatabaseInstance)
        sql.name = 'name'
        sql.settings = create_empty_entity(GcpSqlDBInstanceSettings)
        sql.settings.ip_configuration = create_empty_entity(GcpSqlDBInstanceSettingsIPConfig)
        sql.settings.ip_configuration.require_ssl = False
        context = GcpEnvironmentContext(sql_database_instances=[sql])
        # Act
        result = self.rule.run(context, {})
        # Assert
        self.assertEqual(RuleResultType.FAILED, result.status)
        self.assertEqual(1, len(result.issues))
